@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 #
-# Copyright © 2016 - 2019 Michal Čihař <michal@cihar.com>
+# Copyright © 2012 - 2020 Michal Čihař <michal@cihar.com>
 #
 # This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
 #
@@ -25,7 +24,7 @@ from urllib.parse import urlencode, urlparse
 import dateutil.parser
 import requests
 
-__version__ = "1.2"
+__version__ = "1.4"
 
 URL = "https://weblate.org/"
 DEVEL_URL = "https://github.com/WeblateOrg/wlc"
@@ -39,7 +38,22 @@ class WeblateException(Exception):
     """Generic error."""
 
 
-class Weblate(object):
+class WeblateThrottlingError(WeblateException):
+    def __init__(self):
+        super().__init__("Throttling on the server")
+
+
+class WeblatePermissionError(WeblateException):
+    def __init__(self):
+        super().__init__("You don't have permission to access this object")
+
+
+class WeblateDeniedError(WeblateException):
+    def __init__(self):
+        super().__init__("Access denied, API key is wrong or missing")
+
+
+class Weblate:
     """Weblate API wrapper object."""
 
     def __init__(self, key="", url=API_URL, config=None):
@@ -49,40 +63,43 @@ class Weblate(object):
         else:
             self.key = key
             self.url = url
+        if not self.url.endswith("/"):
+            self.url += "/"
 
     @staticmethod
     def process_error(error):
         """Raise WeblateException for known HTTP errors."""
-        if type(error) is requests.HTTPError:
+        if isinstance(error, requests.HTTPError):
             status_code = error.response.status_code
 
             if status_code == 429:
-                raise WeblateException("Throttling on the server")
-            elif status_code == 404:
+                raise WeblateThrottlingError()
+            if status_code == 404:
                 raise WeblateException(
                     "Object not found on the server "
                     "(maybe operation is not supported on the server)"
                 )
-            elif status_code == 403:
-                raise WeblateException(
-                    "You don't have permission to access this object"
-                )
+            if status_code == 403:
+                raise WeblatePermissionError()
+
+            if status_code == 401:
+                raise WeblateDeniedError()
 
             reason = error.response.reason
             raise WeblateException("HTTP error {0}: {1}".format(status_code, reason))
 
     def raw_request(self, method, path, params=None, files=None):
         """Construct request object and returns raw content."""
-        r = self.invoke_request(method, path, params, files)
+        response = self.invoke_request(method, path, params, files)
 
-        return r.content
+        return response.content
 
     def request(self, method, path, params=None, files=None):
         """Construct request object and returns json response."""
-        r = self.invoke_request(method, path, params, files)
+        response = self.invoke_request(method, path, params, files)
 
         try:
-            return r.json()
+            return response.json()
         except ValueError:
             raise WeblateException("Server returned invalid JSON")
 
@@ -95,7 +112,7 @@ class Weblate(object):
             headers["Authorization"] = "Token {}".format(self.key)
         verify_ssl = self._should_verify_ssl(path)
         try:
-            r = requests.request(
+            response = requests.request(
                 method,
                 path,
                 headers=headers,
@@ -103,11 +120,11 @@ class Weblate(object):
                 verify=verify_ssl,
                 files=files,
             )
-            r.raise_for_status()
+            response.raise_for_status()
         except requests.exceptions.RequestException as error:
             self.process_error(error)
             raise
-        return r
+        return response
 
     def post(self, path, **kwargs):
         """Perform POST request on the API."""
@@ -139,9 +156,9 @@ class Weblate(object):
         parts = path.strip("/").split("/")
         if len(parts) == 3:
             return self.get_translation(path)
-        elif len(parts) == 2:
+        if len(parts) == 2:
             return self.get_component(path)
-        elif len(parts) == 1:
+        if len(parts) == 1:
             return self.get_project(path)
         raise ValueError("Not supported path: {0}".format(path))
 
@@ -177,7 +194,8 @@ class Weblate(object):
         """List languages in the instance."""
         return self.list_factory("languages/", Language)
 
-    def _should_verify_ssl(self, path):
+    @staticmethod
+    def _should_verify_ssl(path):
         """Cheks if it should verify ssl certificates."""
         url = urlparse(path)
         is_localhost = url.netloc.startswith(LOCALHOST_NETLOC)
@@ -193,7 +211,7 @@ class LazyObject(dict):
 
     def __init__(self, weblate, url, **kwargs):
         """Construct object for given Weblate instance."""
-        super(LazyObject, self).__init__()
+        super().__init__()
         self.weblate = weblate
         self._url = url
         self._data = {}
@@ -205,6 +223,12 @@ class LazyObject(dict):
     def get_data(self):
         return copy(self._data)
 
+    def __str__(self):
+        return str(self._data)
+
+    def __repr__(self):
+        return repr(self._data)
+
     def _load_params(self, **kwargs):
         for param in self.PARAMS:
             if param in kwargs:
@@ -244,6 +268,12 @@ class LazyObject(dict):
             self.refresh()
         return self._data[name]
 
+    def setattrvalue(self, name, value):
+        if name not in self.PARAMS:
+            raise AttributeError(name)
+
+        self._data[name] = value
+
     def __getitem__(self, name):
         return self.__getattr__(name)
 
@@ -288,7 +318,7 @@ class LanguageStats(LazyObject):
     ID = "code"
 
 
-class RepoMixin(object):
+class RepoMixin:
     """Repository mixin providing generic repository wide operations."""
 
     def _get_repo_url(self):
@@ -367,6 +397,9 @@ class Project(LazyObject, RepoObjectMixin):
         """Return statistics for translation."""
         self.ensure_loaded("statistics_url")
         data = self.weblate.get(self._attribs["statistics_url"])
+        # Changed in Weblate 4.1:
+        if "url" not in data:
+            data["url"] = self._attribs["statistics_url"]
         return Statistics(weblate=self.weblate, **data)
 
     def languages(self):
@@ -490,6 +523,9 @@ class Translation(LazyObject, RepoObjectMixin):
         """Return statistics for translation."""
         self.ensure_loaded("statistics_url")
         data = self.weblate.get(self._attribs["statistics_url"])
+        # Changed in Weblate 4.1:
+        if "url" not in data:
+            data["url"] = self._attribs["statistics_url"]
         return TranslationStatistics(weblate=self.weblate, **data)
 
     def changes(self):
@@ -525,12 +561,10 @@ class Statistics(LazyObject):
 
     PARAMS = (
         "failing_percent",
-        "url",
         "translated_percent",
         "total_words",
         "failing",
         "translated_words",
-        "url_translate",
         "fuzzy_percent",
         "recent_changes",
         "translated",
