@@ -1,6 +1,6 @@
-# -*- coding: utf-8 -*-
+# PYTHON_ARGCOMPLETE_OK
 #
-# Copyright © 2016 - 2019 Michal Čihař <michal@cihar.com>
+# Copyright © 2012 - 2020 Michal Čihař <michal@cihar.com>
 #
 # This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
 #
@@ -24,7 +24,9 @@ import json
 import logging
 import sys
 from argparse import ArgumentParser
+from datetime import datetime
 
+import argcomplete
 import wlc
 from wlc.config import NoOptionError, WeblateConfig
 
@@ -71,8 +73,10 @@ def get_parser():
     )
     subparser.required = True
 
-    for command in COMMANDS:
-        COMMANDS[command].add_parser(subparser)
+    for command in COMMANDS.values():
+        command.add_parser(subparser)
+
+    argcomplete.autocomplete(parser)
 
     return parser
 
@@ -84,7 +88,15 @@ class CommandError(Exception):
         """Create CommandError exception."""
         if detail is not None:
             message = "\n".join((message, detail))
-        super(CommandError, self).__init__(message)
+        super().__init__(message)
+
+
+class DateTimeEncoder(json.JSONEncoder):
+    def default(self, o):
+        if isinstance(o, datetime):
+            return o.isoformat()
+
+        return super().default(o)
 
 
 def sort_key(value):
@@ -101,7 +113,7 @@ def sorted_items(value):
         yield key, value[key]
 
 
-class Command(object):
+class Command:
     """Basic command object."""
 
     name = ""
@@ -134,18 +146,18 @@ class Command(object):
 
     def print_json(self, value):
         """JSON print."""
-        json.dump(value, self.stdout, indent=2)
+        json.dump(value, self.stdout, cls=DateTimeEncoder, indent=2)
 
     @staticmethod
     def format_value(value):
         """Format value for rendering."""
         if isinstance(value, float):
             return "{0:.1f}".format(value)
-        elif isinstance(value, int):
+        if isinstance(value, int):
             return "{0}".format(value)
-        elif value is None:
+        if value is None:
             return ""
-        elif hasattr(value, "to_value"):
+        if hasattr(value, "to_value"):
             return value.to_value()
         return value
 
@@ -235,7 +247,7 @@ class ObjectCommand(Command):
     @classmethod
     def add_parser(cls, subparser):
         """Create parser for command line."""
-        parser = super(ObjectCommand, cls).add_parser(subparser)
+        parser = super().add_parser(subparser)
         parser.add_argument(
             "object",
             nargs="*",
@@ -272,14 +284,29 @@ class ObjectCommand(Command):
             raise CommandError(message, result["detail"] if "detail" in result else "")
 
 
+class ProjectCommand(ObjectCommand):
+    """Wrapper to allow only project objects."""
+
+    def get_object(self):
+        """Return component object."""
+        obj = super().get_object()
+        if not isinstance(obj, wlc.Project):
+            raise CommandError("Not supported")
+        return obj
+
+    def run(self):
+        """Main execution of the command."""
+        raise NotImplementedError()
+
+
 class ComponentCommand(ObjectCommand):
     """Wrapper to allow only component objects."""
 
     def get_object(self):
         """Return component object."""
-        obj = super(ComponentCommand, self).get_object()
+        obj = super().get_object()
         if not isinstance(obj, wlc.Component):
-            raise CommandError("Not supported")
+            raise CommandError("This command is supported only at component level")
         return obj
 
     def run(self):
@@ -292,9 +319,9 @@ class TranslationCommand(ObjectCommand):
 
     def get_object(self):
         """Return translation object."""
-        obj = super(TranslationCommand, self).get_object()
+        obj = super().get_object()
         if not isinstance(obj, wlc.Translation):
-            raise CommandError("Not supported")
+            raise CommandError("This command is supported only at translation level")
         return obj
 
     def run(self):
@@ -312,7 +339,7 @@ class Version(Command):
     @classmethod
     def add_parser(cls, subparser):
         """Create parser for command line."""
-        parser = super(Version, cls).add_parser(subparser)
+        parser = super().add_parser(subparser)
         parser.add_argument("--bare", action="store_true", help="Print only version")
         return parser
 
@@ -337,15 +364,24 @@ class ListProjects(Command):
 
 
 @register_command
-class ListComponents(Command):
+class ListComponents(ProjectCommand):
     """List components."""
 
     name = "list-components"
-    description = "Lists all components"
+    description = "Lists all components (optionally per project)"
 
     def run(self):
         """Main execution of the command."""
-        self.print(list(self.wlc.list_components()))
+        if self.args.object:
+            obj = self.get_object()
+
+            component_list = list(obj.list())
+            for component in component_list:
+                component.setattrvalue("project", obj)
+
+            self.print(component_list)
+        else:
+            self.print(list(self.wlc.list_components()))
 
 
 @register_command
@@ -361,15 +397,24 @@ class ListLanguages(Command):
 
 
 @register_command
-class ListTranslations(Command):
+class ListTranslations(ComponentCommand):
     """List translations."""
 
     name = "list-translations"
-    description = "Lists all translations"
+    description = "Lists all translations (optionally per component)"
 
     def run(self):
         """Main execution of the command."""
-        self.print(list(self.wlc.list_translations()))
+        if self.args.object:
+            obj = self.get_object()
+
+            translation_list = list(obj.list())
+            for translation in translation_list:
+                translation.setattrvalue("component", obj)
+
+            self.print(translation_list)
+        else:
+            self.print(list(self.wlc.list_translations()))
 
 
 @register_command
@@ -591,7 +636,7 @@ class Download(TranslationCommand):
     @classmethod
     def add_parser(cls, subparser):
         """Create parser for command line."""
-        parser = super(Download, cls).add_parser(subparser)
+        parser = super().add_parser(subparser)
         parser.add_argument(
             "-c", "--convert", help="Convert file format on server (defaults to none)"
         )
@@ -621,7 +666,7 @@ class Upload(TranslationCommand):
     @classmethod
     def add_parser(cls, subparser):
         """Create parser for command line."""
-        parser = super(Upload, cls).add_parser(subparser)
+        parser = super().add_parser(subparser)
         parser.add_argument("-i", "--input", help="File to upload (defaults to stdin)")
         parser.add_argument(
             "--overwrite",
@@ -660,7 +705,18 @@ class Upload(TranslationCommand):
         else:
             result = obj.upload(self.stdin.buffer.read(), **kwargs)
 
-        self.check_result(result, "Failed to upload translations!")
+        if not (
+            "count" in result
+            and "result" in result
+            and "total" in result
+            and "accepted" in result
+            and "not_found" in result
+            and "skipped" in result
+        ):
+            raise CommandError(
+                "Failed to upload translations!",
+                result["detail"] if "detail" in result else "",
+            )
 
 
 def parse_settings(args, settings):
@@ -702,6 +758,19 @@ def main(settings=None, stdout=None, stdin=None, args=None):
     try:
         command.run()
         return 0
+    except wlc.WeblateDeniedError:
+        url, key = config.get_url_key()
+        if key:
+            print(
+                f"API key configured for {url} was rejected by server.", file=sys.stderr
+            )
+        else:
+            print(f"Missing API key for {url}.", file=sys.stderr)
+            print(
+                f"The API key can be specified by --key or in the configuration file.",
+                file=sys.stderr,
+            )
+        return 1
     except (CommandError, wlc.WeblateException) as error:
         print("Error: {0}".format(error), file=sys.stderr)
         return 1
