@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 #
-# Copyright © 2016 - 2019 Michal Čihař <michal@cihar.com>
+# Copyright © 2012 - 2020 Michal Čihař <michal@cihar.com>
 #
 # This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
 #
@@ -23,14 +22,16 @@ import os
 import re
 from unittest import TestCase
 
-import httpretty
+import responses
+from requests.exceptions import RequestException
+
 from requests_toolbelt.multipart import decoder
 
 DATA_TEST_BASE = os.path.join(os.path.dirname(__file__), "test_data", "api")
 
 
-class ResponseHandler(object):
-    """httpretty response handler."""
+class ResponseHandler:
+    """responses response handler."""
 
     def __init__(self, body, filename, auth=False):
         """Construct response handler object."""
@@ -38,14 +39,14 @@ class ResponseHandler(object):
         self.filename = filename
         self.auth = auth
 
-    def __call__(self, request, uri, headers):
-        """Call interface for httpretty."""
-        if self.auth and request.headers["Authorization"] != "Token KEY":
-            return 403, headers, ""
+    def __call__(self, request):
+        """Call interface for responses."""
+        if self.auth and request.headers.get("Authorization") != "Token KEY":
+            return 403, {}, ""
 
         content = self.get_content(request)
 
-        return 200, headers, content
+        return 200, {}, content
 
     def get_content(self, request):
         """Return content for given request."""
@@ -66,13 +67,10 @@ class ResponseHandler(object):
                 "multipart/form-data"
             ):
                 return self.get_multipart_filename(content_type, request)
-            else:
-                return "--".join(
-                    (self.filename, request.method, request.body.decode("ascii"))
-                )
-        elif "?" in request.path:
+            return "--".join((self.filename, request.method, request.body or ""))
+        if "?" in request.path_url:
             return "--".join(
-                (self.filename, request.method, request.path.split("?", 1)[-1])
+                (self.filename, request.method, request.path_url.split("?", 1)[-1])
             )
         return None
 
@@ -101,9 +99,9 @@ class ResponseHandler(object):
     @staticmethod
     def get_multipart_name(content_disposition):
         """Return multipart name from content disposition."""
-        m = re.search(r'name\s*=\s*"(?P<name>[A-Za-z]+)"', content_disposition)
-
-        return m.group("name")
+        return re.search(
+            r'name\s*=\s*"(?P<name>[A-Za-z]+)"', content_disposition
+        ).group("name")
 
 
 def register_uri(path, domain="http://127.0.0.1:8000/api", auth=False):
@@ -111,40 +109,44 @@ def register_uri(path, domain="http://127.0.0.1:8000/api", auth=False):
     filename = os.path.join(DATA_TEST_BASE, path.replace("/", "-"))
     url = "/".join((domain, path, ""))
     with open(filename, "rb") as handle:
-        httpretty.register_uri(
-            httpretty.GET,
+        responses.add_callback(
+            responses.GET,
             url,
-            body=ResponseHandler(handle.read(), filename, auth),
+            callback=ResponseHandler(handle.read(), filename, auth),
             content_type="application/json",
         )
-        httpretty.register_uri(
-            httpretty.POST,
+        responses.add_callback(
+            responses.POST,
             url,
-            body=ResponseHandler(handle.read(), filename, auth),
+            callback=ResponseHandler(handle.read(), filename, auth),
             content_type="application/json",
         )
-        httpretty.register_uri(
-            httpretty.DELETE,
+        responses.add_callback(
+            responses.DELETE,
             url,
-            body=ResponseHandler(handle.read(), filename, auth),
+            callback=ResponseHandler(handle.read(), filename, auth),
             content_type="application/json",
         )
 
 
-def raise_error(request, uri, headers):
+def raise_error(request):
     """Raise IOError."""
-    # pylint: disable=W0613
-    raise IOError("Some error")
+    if "/io" in request.path_url:
+        raise RequestException("Some error")
+    raise Exception("Bug")
 
 
-def register_error(path, code, domain="http://127.0.0.1:8000/api", body=None):
+def register_error(path, code, domain="http://127.0.0.1:8000/api", **kwargs):
     """Simplified URL error registration."""
     url = "/".join((domain, path, ""))
-    httpretty.register_uri(httpretty.GET, url, body=body, status=code)
+    if "callback" in kwargs:
+        responses.add_callback(responses.GET, url, **kwargs)
+    else:
+        responses.add(responses.GET, url, status=code, **kwargs)
 
 
 def register_uris():
-    """Register URIs for httpretty."""
+    """Register URIs for responses."""
     paths = (
         "changes",
         "projects",
@@ -183,18 +185,20 @@ def register_uris():
     register_error("projects/denied", 403)
     register_error("projects/throttled", 429)
     register_error("projects/error", 500)
-    register_error("projects/io", 500, body=raise_error)
+    register_error("projects/io", 500, callback=raise_error)
+    register_error("projects/bug", 500, callback=raise_error)
+    register_error("projects", 401, domain="http://denied.example.com")
 
 
 class APITest(TestCase):
     """Base class for API testing."""
 
     def setUp(self):
-        """Enable httpretty and register urls."""
-        httpretty.enable()
+        """Enable responses and register urls."""
+        responses.mock.start()
         register_uris()
 
     def tearDown(self):
-        """Disable httpretty."""
-        httpretty.disable()
-        httpretty.reset()
+        """Disable responses."""
+        responses.mock.stop()
+        responses.mock.reset()
